@@ -74,7 +74,7 @@ static int _setup_egl(RfConverter *this)
 	EGLint n;
 	EGLint size;
 	EGLConfig config = NULL;
-	EGLConfig *configs;
+	g_autofree EGLConfig *configs = NULL;
 	// clang-format off
 	EGLint config_attribs[] = {
 		EGL_RENDERABLE_TYPE, EGL_OPENGL_ES3_BIT,
@@ -155,8 +155,6 @@ static int _setup_egl(RfConverter *this)
 		return -3;
 	}
 
-	g_free(configs);
-
 	return 0;
 }
 
@@ -537,6 +535,7 @@ out:
 	if (ret < 0) {
 		_clean_gl(this);
 		_clean_egl(this);
+		g_clear_pointer(&this->card_path, g_free);
 	}
 	return ret;
 }
